import socket
import logging

class StatsdSender(object):
    def __init__(self, host, port):
        self.sock = socket.socket(
            socket.AF_INET,
            socket.SOCK_DGRAM
        )
        self.dest = (host, port)

    def send(self, data):
        logging.debug("Sending data: %s", data)
        self.sock.sendto(data.encode(), self.dest)



import re
import logging

class Mangler(object):
    def __init__(self, patterns, sender):
        self.patterns = []
        for pattern in patterns:
            self.patterns.append(
                {
                    "search": re.compile(pattern["search"]),
                    "replace": pattern["replace"]
                }
            )
        self.sender = sender

    def mangle(self, metrics):
        logging.debug("Mangling")
        for metric in metrics.split("\n"):
            for pattern in self.patterns:
                metric = pattern["search"].sub(pattern["replace"], metric)
            logging.debug("Sending metric %s", metric)
            self.sender.send(metric)



import logging
import time
import toml
import sys
import os.path
from statsd_mangler import StatsdListener
from statsd_mangler import StatsdSender
from statsd_mangler import Mangler

def main():
    if len(sys.argv) < 2:
        print("statsd_mangler <statsd_mangler.toml>")
        exit(1)
    configFile = sys.argv[1]
    if not os.path.isfile(configFile):
        print("Config file {} not found".format(configFile))
    config = toml.load(configFile)
    logging.basicConfig(
        filename=config['log']['destination'],
        format="%(asctime)s %(levelname)s %(message)s"
    )
    logging.getLogger().setLevel(config["log"]["level"])
    logging.info("Starting up")
    logging.debug("Config loaded: %s", config)

    sender = StatsdSender(config["destination"]["host"], config["destination"]["port"])
    mangler = Mangler(config["patterns"], sender)
    listener = StatsdListener(config["listen"]["port"], mangler)

    while True:
        try:
            listener.listen()
        except Exception:
            logging.exception(Exception)
            time.sleep(10)

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name="statsd_mangler",
    version="0.1.1",
    packages=find_packages(),
    install_requires=['toml>=0.10.0'],
    entry_points = {
        'console_scripts': ['statsd_mangler=statsd_mangler.__main__:main']
    },
    author="Carl Flippin",
    author_email="cflippin@opentable.com",
    description="Namespace mangler for statsd metrics",
    license="MIT",
    url="https://github.com/opentable/statsd_mangler"
)



import socket
import select
import logging

class StatsdListener(object):
    def __init__(self, port, mangler):
        self.sock = socket.socket(
            socket.AF_INET,
            socket.SOCK_DGRAM
        )
        self.sock.bind(("", port))
        self.mangler = mangler

    def listen(self):
        logging.debug("Starting listener")
        while True:
            ready = select.select([self.sock], [], [], 1)
            if ready[0]:
                data = self.sock.recv(4096)
                self.mangler.mangle(data.decode().strip())
